<?php

namespace Drupal\single_page_protection\EventSubscriber;

use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Symfony\Component\HttpKernel\KernelEvents;
use Symfony\Component\HttpKernel\Event\RequestEvent;
use Drupal\Core\Routing\RouteMatchInterface;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Drupal\Core\Url;

class PageRequestSubscriber implements EventSubscriberInterface {

    /**
     * Checks if the current page is protected and manages access accordingly.
     * If the page is protected and the password has not been entered in the session, redirects to the password form.
     * Otherwise, allows normal access to the page.
     */
    public function checkProtectedPage(RequestEvent $event) {
        $request = $event->getRequest();
        $current_path = $request->getPathInfo();

        // Check for information about the protected page.
        $protected_page = $this->getProtectedPageInfo($current_path);

        // First, check if the password for this URL has already been entered in the session.
        if ($protected_page && empty($_SESSION['single_page_protection'][$current_path])) {
            // If the password has not been entered before, redirect the user to the password form.
            $password_form_url = Url::fromRoute('single_page_protection.password_form', ['path' => $current_path])->toString();
            $response = new RedirectResponse($password_form_url);
            $event->setResponse($response);
        }
        // If the password has been entered before, this block will not execute, and the user will be directed to the protected page directly.
    }

    /**
     * Checks the database for information on the protected page.
     * This includes checking if the URL is marked as protected and retrieving the associated password if it is.
     */
    private function getProtectedPageInfo($path) {
        $result = \Drupal::database()->select('single_page_protection', 'n')
            ->fields('n', ['url', 'password'])
            ->condition('url', $path)
            ->execute()
            ->fetchAssoc();

        return $result ? $result : NULL;
    }

    /**
     * Registers the event subscriber with the Drupal event dispatcher.
     * This method subscribes to the REQUEST event to intercept page requests and check if they are protected.
     */
    public static function getSubscribedEvents() {
        $events[KernelEvents::REQUEST][] = ['checkProtectedPage', 100];
        return $events;
    }
}
